@@ -130,7 +130,7 @@ fn test_transaction_spend_in_same_block() {
 
     let last_cellbase = &shared.consensus().genesis_block().transactions()[1];
     let last_cellbase_hash = last_cellbase.hash();
-    let tx1 = create_multi_outputs_transaction(&last_cellbase, vec![0], 2, vec![1]);
+    let tx1 = create_multi_outputs_transaction(last_cellbase, vec![0], 2, vec![1]);
     let tx1_hash = tx1.hash();
     let tx2 = create_multi_outputs_transaction(&tx1, vec![0], 2, vec![2]);
     let tx2_hash = tx2.hash();
@@ -255,7 +255,7 @@ fn test_transaction_conflict_in_different_blocks() {
     chain.gen_empty_block(&mock_store);
 
     let last_cellbase = &shared.consensus().genesis_block().transactions()[1];
-    let tx1 = create_multi_outputs_transaction(&last_cellbase, vec![0], 2, vec![1]);
+    let tx1 = create_multi_outputs_transaction(last_cellbase, vec![0], 2, vec![1]);
     let tx1_hash = tx1.hash();
     let tx2 = create_multi_outputs_transaction(&tx1, vec![0], 2, vec![1]);
     let tx3 = create_multi_outputs_transaction(&tx1, vec![0], 2, vec![2]);
